import React from 'react';
import ReactDOM from 'react-dom';
import { range, scaleBand } from 'd3';

// Production rules for the L-System.
const rules = {
  A: 'ABA',
  B: 'BBB'
};

const initialState = 'A';
const iterations = 8;
const L = str =>
  Array.from(str)
    .map(d => rules[d])
    .join('');
const iterate = i => (i ? L(iterate(i - 1)) : initialState);
const data = range(iterations).map(iterate);

// console.log(JSON.stringify(data, null, 2));
// Prints the following:
// [
//   "A",
//   "ABA",
//   "ABABBBABA",
//   "ABABBBABABBBBBBBBBABABBBABA",
//   "ABABBBABABBBBBBBBBABABBBABABBBBBBBBBBBBBBBBBBBBBBBBBBBABABBBABABBBBBBBBBABABBBABA",
//   ...

const width = 960;
const height = 500;
const x = scaleBand().range([0, width]);
const y = d3
  .scaleBand()
  .domain(d3.range(iterations))
  .rangeRound([0, height]);
const rects = data
  .map((str, j) => {
    x.domain(d3.range(str.length));
    return Array.from(str)
      .map((symbol, i) => ({
        symbol: symbol,
        x: x(i),
        y: y(j),
        width: x.bandwidth(),
        height: y.bandwidth() + 1 // Add 1 to get rid of subpixel gaps
      }))
      .filter(d => d.symbol === 'A');
  })
  .reduce((a, b) => a.concat(b), []);

const App = () => (
  <svg width={width} height={height}>
    {rects.map(d => (
      <rect {...d} />
    ))}
  </svg>
);

ReactDOM.render(<App />, document.getElementById('root'));
